@@ -243,6 +243,8 @@ impl Class {
                 }
                 impl ::std::convert::From<&#from> for #into {
                     fn from(value: &#from) -> Self {
+                        // This unwrap is legitimate because conversion to base can never fail because 
+                        // the base can never change across versions.
                         ::windows::Interface::cast(value).unwrap()
                     }
                 }
@@ -251,7 +253,7 @@ impl Class {
                         ::windows::Param::Owned(::std::convert::Into::<#into>::into(self))
                     }
                 }
-                impl<'a> ::windows::IntoParam<'a, #into> for &'a #from {
+                impl<'a> ::windows::IntoParam<'a, #into> for &#from {
                     fn into_param(self) -> ::windows::Param<'a, #into> {
                         ::windows::Param::Owned(::std::convert::Into::<#into>::into(::std::clone::Clone::clone(self)))
                     }
