@@ -34,14 +34,15 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                     type IntoIter = IIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
-                        self.First().unwrap()
+                        ::std::iter::IntoIterator::into_iter(&self)
                     }
                 }
-                impl<'a, T: ::windows::RuntimeType> ::std::iter::IntoIterator for &'a IIterable<T> {
+                impl<T: ::windows::RuntimeType> ::std::iter::IntoIterator for &IIterable<T> {
                     type Item = T;
                     type IntoIter = IIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
+                        // TODO: not sure how to avoid this unwrap, although it should always succeed in practice.
                         self.First().unwrap()
                     }
                 }
@@ -51,15 +52,13 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
         TypeName::IVectorView => {
             return quote! {
                 pub struct VectorViewIterator<T: ::windows::RuntimeType + 'static> {
-                    vector: IVectorView<T>,
+                    vector: ::std::option::Option<IVectorView<T>>,
                     current: u32,
-                    size: u32,
                 }
 
                 impl<T: ::windows::RuntimeType> VectorViewIterator<T> {
-                    pub fn new(vector: IVectorView<T>) -> Self {
-                        let size = vector.Size().unwrap();
-                        Self { vector, current: 0, size }
+                    pub fn new(vector: ::std::option::Option<IVectorView<T>>) -> Self {
+                        Self { vector, current: 0 }
                     }
                 }
 
@@ -67,13 +66,14 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                     type Item = T;
 
                     fn next(&mut self) -> ::std::option::Option<Self::Item> {
-                        if self.current >= self.size {
-                            return None;
-                        }
-
-                        let result = self.vector.GetAt(self.current);
-                        self.current += 1;
-                        result.ok()
+                        self.vector.as_ref()
+                            .and_then(|vector| {
+                                vector.GetAt(self.current).ok()
+                            })
+                            .and_then(|result| {
+                                self.current += 1;
+                                Some(result)
+                            })
                     }
                 }
 
@@ -82,15 +82,16 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                     type IntoIter = VectorViewIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
-                        VectorViewIterator::new(self)
+                        ::std::iter::IntoIterator::into_iter(&self)
                     }
                 }
-                impl<'a, T: ::windows::RuntimeType> ::std::iter::IntoIterator for &'a IVectorView<T> {
+                impl<T: ::windows::RuntimeType> ::std::iter::IntoIterator for &IVectorView<T> {
                     type Item = T;
                     type IntoIter = VectorViewIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
-                        VectorViewIterator::new(::std::clone::Clone::clone(self))
+                        // TODO: shouldn't need to clone - VectorViewIterator should hold a reference
+                        VectorViewIterator::new(::std::option::Option::Some(::std::clone::Clone::clone(self)))
                     }
                 }
             };
@@ -98,15 +99,13 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
         TypeName::IVector => {
             return quote! {
                 pub struct VectorIterator<T: ::windows::RuntimeType + 'static> {
-                    vector: IVector<T>,
+                    vector: ::std::option::Option<IVector<T>>,
                     current: u32,
-                    size: u32,
                 }
 
                 impl<T: ::windows::RuntimeType> VectorIterator<T> {
-                    pub fn new(vector: IVector<T>) -> Self {
-                        let size = vector.Size().unwrap();
-                        Self { vector, current: 0, size }
+                    pub fn new(vector: ::std::option::Option<IVector<T>>) -> Self {
+                        Self { vector, current: 0 }
                     }
                 }
 
@@ -114,13 +113,14 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                     type Item = T;
 
                     fn next(&mut self) -> ::std::option::Option<Self::Item> {
-                        if self.current >= self.size {
-                            return None;
-                        }
-
-                        let result = self.vector.GetAt(self.current);
-                        self.current += 1;
-                        result.ok()
+                        self.vector.as_ref()
+                            .and_then(|vector| {
+                                vector.GetAt(self.current).ok()
+                            })
+                            .and_then(|result| {
+                                self.current += 1;
+                                Some(result)
+                            })
                     }
                 }
 
@@ -129,15 +129,16 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                     type IntoIter = VectorIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
-                        VectorIterator::new(self)
+                        ::std::iter::IntoIterator::into_iter(&self)
                     }
                 }
-                impl<'a, T: ::windows::RuntimeType> ::std::iter::IntoIterator for &'a IVector<T> {
+                impl<T: ::windows::RuntimeType> ::std::iter::IntoIterator for &IVector<T> {
                     type Item = T;
                     type IntoIter = VectorIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
-                        VectorIterator::new(::std::clone::Clone::clone(self))
+                        // TODO: shouldn't need to clone - VectorIterator should hold a reference
+                        VectorIterator::new(::std::option::Option::Some(::std::clone::Clone::clone(self)))
                     }
                 }
             };
@@ -163,15 +164,15 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                         type IntoIter = #wfc VectorViewIterator<Self::Item>;
 
                         fn into_iter(self) -> Self::IntoIter {
-                            #wfc VectorViewIterator::new(self.into())
+                            ::std::iter::IntoIterator::into_iter(&self)
                         }
                     }
-                    impl<'a, #constraints> ::std::iter::IntoIterator for &'a #name {
+                    impl<#constraints> ::std::iter::IntoIterator for &#name {
                         type Item = #item;
                         type IntoIter = #wfc VectorViewIterator<Self::Item>;
 
                         fn into_iter(self) -> Self::IntoIter {
-                            #wfc VectorViewIterator::new(self.into())
+                            #wfc VectorViewIterator::new(::std::convert::TryInto::try_into(self).ok())
                         }
                     }
                 };
@@ -187,15 +188,15 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                         type IntoIter = #wfc VectorIterator<Self::Item>;
 
                         fn into_iter(self) -> Self::IntoIter {
-                            #wfc VectorIterator::new(self.into())
+                            ::std::iter::IntoIterator::into_iter(&self)
                         }
                     }
-                    impl<'a, #constraints> ::std::iter::IntoIterator for &'a #name {
+                    impl<#constraints> ::std::iter::IntoIterator for &#name {
                         type Item = #item;
                         type IntoIter = #wfc VectorIterator<Self::Item>;
 
                         fn into_iter(self) -> Self::IntoIter {
-                            #wfc VectorIterator::new(self.into())
+                            #wfc VectorIterator::new(::std::convert::TryInto::try_into(self).ok())
                         }
                     }
                 };
@@ -220,14 +221,15 @@ pub fn gen_iterator(def: &TypeDef, interfaces: &[InterfaceInfo], gen: &Gen) -> T
                     type IntoIter = #wfc IIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
-                        self.First().unwrap()
+                        ::std::iter::IntoIterator::into_iter(&self)
                     }
                 }
-                impl<'a, #constraints> ::std::iter::IntoIterator for &'a #name {
+                impl<#constraints> ::std::iter::IntoIterator for &#name {
                     type Item = #item;
                     type IntoIter = #wfc IIterator<Self::Item>;
 
                     fn into_iter(self) -> Self::IntoIter {
+                        // TODO: not sure how to avoid this unwrap, although it should always succeed in practice.
                         self.First().unwrap()
                     }
                 }
